# a http server
import cherrypy

# html template
from mako.lookup import TemplateLookup

# sys enables dynamic function call
import os, sys

# this does nothing other than show some file statistics
def upload(myFile):

        out = """<html>
        <body>
            myFile length: %s<br />
            myFile filename: %s<br />
            myFile mime-type: %s
        </body>
        </html>"""

        # Although this just counts the file length, it demonstrates
        # how to read large files in chunks instead of all at once.
        # CherryPy reads the uploaded file into a temporary file;
        # myFile.file.read reads from that.

        size = 0
        while True:
            data = myFile.file.read(8192)
            if not data:
                break
            size += len(data)

        return out % (size, myFile.filename, myFile.content_type)

# image class deals with image processing
class image:
	
	@cherrypy.expose
	def index(self):
		html_dir = os.path.join(os.path.dirname(__file__), 'html')
		lookup = TemplateLookup(directories=[html_dir])
		# mako really makes html generation simple
		return lookup.get_template('process.html').render(contype="image")

	# a method is translated to a subfolder /image/upload
	@cherrypy.expose
	def upload(self, myFile):
		return upload(myFile) 

# sound class handles sound processing
class sound:

	@cherrypy.expose
	def index(self):
		html_dir = os.path.join(os.path.dirname(__file__), 'html')
		lookup = TemplateLookup(directories=[html_dir])
		return lookup.get_template('process.html').render(contype="sound")
	
	@cherrypy.expose
	def upload(self, myFile):
		return upload(myFile) 

# start the http server
def run(utils):

	# mount the main page
	cherrypy.tree.mount(server())

	# mount the handlers specified by utils
	for uid in utils: 
		cherrypy.log("loading", context="%s" % uid)
		obj = getattr(sys.modules[__name__], "%s" % uid)()
		# this enables hyperlink click in html 
		cherrypy.tree.mount(obj, script_name='/%s' % uid) 	
		

# server class
class server:

	# the available services
	def __init__(self):
		util_dir = os.path.join(os.path.dirname(__file__), 'utility')
		self.utils = os.listdir(util_dir)

	# the main entry
	@cherrypy.expose
	def index(self):
		html_dir = os.path.join(os.path.dirname(__file__), 'html')
		lookup = TemplateLookup(directories=[html_dir])
		return lookup.get_template('index.html').render(args=self.utils)



if __name__ == '__main__':

	run(server().utils)
	cherrypy.engine.start()
	cherrypy.engine.block()
